package at.ac.uibk.gitsearch.service;

import static org.junit.Assert.assertEquals;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.repository.search.testESService.ElasticSearchTestConfiguration;
import at.ac.uibk.gitsearch.service.dto.StatisticsDTO;
import java.io.IOException;
import java.util.concurrent.TimeUnit;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.node.NodeValidationException;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.Timeout;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.security.test.context.support.WithMockUser;

@SpringBootTest(classes = GitsearchApp.class)
@WithMockUser(value = "test", authorities = "sharing")
public class AchievementServiceIT {

    @SuppressWarnings("unused")
    private static final Logger LOGGER = LogManager.getLogger(AchievementServiceIT.class);

    @Autowired
    RestHighLevelClient elasticsearchTestClient;

    @Autowired
    ElasticSearchTestConfiguration elasticSearchTestConfiguration;

    static ElasticSearchTestConfiguration staticElasticSearchTestConfiguration;

    @BeforeEach // must be started as BeforeEach, in order to autowire the ElasticSearchTestConfiguration
    @Timeout(value = 2, unit = TimeUnit.MINUTES)
    public void setUpESServer() throws IOException, NodeValidationException {
        if (staticElasticSearchTestConfiguration == null) {
            elasticSearchTestConfiguration.startTestNode();
            staticElasticSearchTestConfiguration = elasticSearchTestConfiguration;
        }
    }

    @AfterAll
    public static void shutDownESServer() {
        if (staticElasticSearchTestConfiguration != null) {
            staticElasticSearchTestConfiguration.stopTestContainer();
            staticElasticSearchTestConfiguration = null;
        }
    }

    @BeforeEach
    public void setUpIndex() {}

    @Autowired
    AchievementService achievementService;

    @Autowired
    SearchService searchService;

    @Autowired
    private StatisticsService statisticsService;

    @Test
    public void testSearchUserStatisticsEmpty() throws IOException {
        final StatisticsDTO result = achievementService.searchUserStatistics(" ");
        assertEquals(Integer.valueOf(0), result.getDownloads());
        assertEquals(Integer.valueOf(0), result.getViews());
    }

    @Test
    @Timeout(value = 2, unit = TimeUnit.MINUTES)
    public void testSearchUserStatisticsValid() throws IOException {
        StatisticsDTO newStats = new StatisticsDTO();
        newStats.setDownloads(1);
        newStats.setViews(1);
        newStats.setExerciseID("[143]");
        statisticsService.save(newStats);

        final StatisticsDTO result = achievementService.searchUserStatistics("stefan.podlipnig@tuwien.ac.at");
        assertEquals(Integer.valueOf(1), result.getDownloads());
        assertEquals(Integer.valueOf(1), result.getViews());
    }
}
