import { TestBed, getTestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';
import moment from 'moment';
import { DATE_FORMAT } from 'app/app.constants';
import { LikesService } from 'app/entities/likes/likes.service';
import { ILikes, Likes } from 'app/shared/model/likes.model';

describe('Service Tests', () => {
  describe('Likes Service', () => {
    let injector: TestBed;
    let service: LikesService;
    let httpMock: HttpTestingController;
    let elemDefault: ILikes;
    let expectedResult: ILikes | ILikes[] | boolean | null;
    let currentDate: moment.Moment;

    beforeEach(() => {
      TestBed.configureTestingModule({
        imports: [HttpClientTestingModule],
      });
      expectedResult = null;
      injector = getTestBed();
      service = injector.get(LikesService);
      httpMock = injector.get(HttpTestingController);
      currentDate = moment();

      elemDefault = new Likes(0, currentDate, 0, '0');
    });

    describe('Service methods', () => {
      it('should find an element', () => {
        const returnedFromService = Object.assign(
          {
            date: currentDate.format(DATE_FORMAT),
          },
          elemDefault
        );

        service.find(123).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'GET' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(elemDefault);
      });

      it('should create a Likes', () => {
        const returnedFromService = Object.assign(
          {
            id: 0,
            date: currentDate.format(DATE_FORMAT),
          },
          elemDefault
        );

        const expected = Object.assign(
          {
            date: currentDate,
          },
          returnedFromService
        );

        service.create(new Likes()).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'POST' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(expected);
      });

      it('should update a Likes', () => {
        const returnedFromService = Object.assign(
          {
            date: currentDate.format(DATE_FORMAT),
            userID: 1,
            projectID: 1,
          },
          elemDefault
        );

        const expected = Object.assign(
          {
            date: currentDate,
          },
          returnedFromService
        );

        service.update(expected).subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'PUT' });
        req.flush(returnedFromService);
        expect(expectedResult).toMatchObject(expected);
      });

      it('should return a list of Likes', () => {
        const returnedFromService = Object.assign(
          {
            date: currentDate.format(DATE_FORMAT),
            userID: 1,
            projectID: 1,
          },
          elemDefault
        );

        const expected = Object.assign(
          {
            date: currentDate,
          },
          returnedFromService
        );

        service.query().subscribe(resp => (expectedResult = resp.body));

        const req = httpMock.expectOne({ method: 'GET' });
        req.flush([returnedFromService]);
        httpMock.verify();
        expect(expectedResult).toContainEqual(expected);
      });

      it('should delete a Likes', () => {
        service.delete(123).subscribe(resp => (expectedResult = resp.ok));

        const req = httpMock.expectOne({ method: 'DELETE' });
        req.flush({ status: 200 });
        expect(expectedResult);
      });

      it('like flow', () => {
        service.likeExercise('123').subscribe(resp => (expectedResult = resp.ok));
        const req = httpMock.expectOne({ method: 'PUT' });
        req.flush({ status: 200 });
        let returnVal = false;
        service.hasLiked('123').subscribe(resp => (returnVal = resp));
        const req2 = httpMock.expectOne({ method: 'GET' });
        req2.flush({ status: 200 });
        expect(returnVal);

        service.getLikesForexerciseID('123').subscribe();
        const req3 = httpMock.expectOne({ method: 'GET' });
        req3.flush({ status: 200 });

        service.unlikeExercise('123').subscribe(resp => (expectedResult = resp.ok));
        const req4 = httpMock.expectOne({ method: 'DELETE' });
        req4.flush({ status: 200 });
        service.hasLiked('123').subscribe(resp => (returnVal = resp));
        const req5 = httpMock.expectOne({ method: 'GET' });
        req5.flush({ status: 200 });
        expect(!returnVal);
      });
    });

    afterEach(() => {
      httpMock.verify();
    });
  });
});
