import Spy = jasmine.Spy;
import { of } from 'rxjs';

import { SpyObject } from './spyobject';
import { UserWatchListService } from 'app/entities/user-watch-list/user-watch-list.service';
import { IUserWatchList, UserWatchList } from 'app/shared/model/user-watch-list.model';
import { CheckFrequency } from 'app/shared/model/enumerations/check-frequency.model';
import { HttpHeaders, HttpResponse } from '@angular/common/http';

export class MockUserWatchListService extends SpyObject {
  getSpy: Spy;
  findMyWatchlistsSpy: Spy;
  querySpy: Spy;
  deleteSpy: Spy;
  updateSpy: Spy;
  createSpy: Spy;

  defaultWatchList: IUserWatchList[] = [
    {
      id: 1,
      name: 'My WatchList',
      checkFrequency: CheckFrequency.NEVER,
      userLogin: 'any',
      userId: 0,
    },
  ];

  constructor() {
    super(UserWatchListService);

    const entity = new UserWatchList(123);

    this.getSpy = this.spy('get').andReturn(this);
    this.findMyWatchlistsSpy = this.spy('findMyWatchlists').andReturn(of({ body: this.defaultWatchList }));

    const headers = new HttpHeaders().append('link', 'link;link');
    this.querySpy = this.spy('query').and.returnValue(of(new HttpResponse({ body: [entity], headers })));

    this.deleteSpy = this.spy('delete').and.returnValue(of({}));

    this.updateSpy = this.spy('update').and.returnValue(of(new HttpResponse({ body: entity })));

    const newEntity = new UserWatchList();

    this.createSpy = this.spy('create').and.returnValue(of(new HttpResponse({ body: newEntity })));
  }
}
